import os
from zheinit.utils import show_available_version, run_cmd, echo


def init():
    if "bash" not in run_cmd("echo $SHELL")[0]:
        if "bash" not in run_cmd("cat /etc/shells")[0]:
            echo("未找到 bash 环境，请先安装 bash 环境！", "red")
        else:        
            echo("请在 bash 环境下运行本工具！")
        echo("您可以通过以下命令查看所支持的 Shell 类型：\ncat /etc/shells", "red")
        echo("您可以通过以下命令切换 Shell 类型：\nchsh -s /bin/bash", "red")
        exit()

    pkg_current_path = os.path.dirname(os.path.abspath(__file__))
    python_version = "3.9"
    env_name = "zhei"

    # 读取 ~/.bashrc 文件内容
    if not os.path.exists("~/.bashrc"):
        run_cmd("touch ~/.bashrc")
    bashrc = run_cmd("cat ~/.bashrc")[0]
    if "zhei-init" not in bashrc:
        print("未找到 zhei-init 配置，即将注入配置到 ~/.bashrc（完成后可能需要重启初始化工具）")
        # ---------------------------------------------------------------------------- #
        #                         配置 Bash 环境变量                                     
        # ---------------------------------------------------------------------------- #
        bash = [
            "cd ~/",
            "cat {}/bash_config.txt >> ~/.bashrc".format(pkg_current_path),
            "source ~/.bashrc",
        ]
        run_cmd(bash)
        
    echo("")
    echo("")
    echo("")
    echo("      _____                          _      ____                  _")
    echo("     |  __ \\                        | |    |  _ \\                | |")
    echo("     | |__) |____      _____ _ __ __| |    | |_) |_   _          | |")
    echo("     |  ___/ _ \\ \\ /\\ / / _ \\ '__/ _` |    |  _ <| | | |     _   | |")
    echo("     | |  | (_) \\ V  V /  __/ | | (_| |    | |_) | |_| |    | |__| |")
    echo("     |_|   \\___/ \\_/\\_/ \\___|_|  \\__,_|    |____/ \\__, |     \\____/")
    echo("                                                   __/ |")
    echo("                                                  |___/")
    echo("")
    echo("")
    echo("")
    echo("             欢迎使用服务器环境初始化工具 zhei-init ！", "green")
    echo("")
    echo("   本工具将会帮助您初始化服务器环境，下面是功能菜单，可输入序号进行配置", "green")

    step = "-1"
    while step != "0":
        if step != "-1":
            echo("\n是否继续配置？（y/n）", "yellow")
            conti = input()
            if conti != "y":
                break
        echo(" ", "blue")
        echo("1、设置 pip 源", "blue") 
        echo("2、安装 MiniConda", "blue")
        echo("3、安装 ranger 并自动配置", "blue")
        echo("4、创建 Conda  Pytorch 环境", "blue")
        echo("5、安装 Redis", "blue")
        echo("6、生成公钥", "blue")
        echo("7、生成 zhei 项目模板", "blue")
        echo("0、退出", "blue")
        echo(" ", "blue")
        echo("请在下方输入操作序号：", "yellow")
        step = input()
        if step == "1":
            # ---------------------------------------------------------------------------- #
            #                         设置 pip 源                                     
            # ---------------------------------------------------------------------------- #
            pip_source = [
                "conda config --add channels https://mirrors.tuna.tsinghua.edu.cn/anaconda/pkgs/free/",
                "conda config --add channels https://mirrors.tuna.tsinghua.edu.cn/anaconda/pkgs/main/",
                "conda config --set show_channel_urls yes",
                "pip config set global.index-url https://pypi.tuna.tsinghua.edu.cn/simple",
                "pip config set global.extra-index-url https://pypi.org/simple"
            ]
            run_cmd(pip_source)
        
        elif step == "2":
            # ---------------------------------------------------------------------------- #
            #                         安装 MiniConda                                     
            # ---------------------------------------------------------------------------- #
            run_cmd([
                "wget https://repo.anaconda.com/miniconda/Miniconda3-latest-Linux-x86_64.sh",
                "sh Miniconda3-latest-Linux-x86_64.sh",
            ])

        elif step == "3":
            # ---------------------------------------------------------------------------- #
            #                             安装 ranger                                 
            # ---------------------------------------------------------------------------- #
            ranger = [
                "pip install ranger-fm",
                "mv {}/ranger ~/.config/".format(pkg_current_path),
            ]
            run_cmd(ranger)

        elif step == "4":
            # ---------------------------------------------------------------------------- #
            #                         创建 zhei 环境                                     
            # ---------------------------------------------------------------------------- #
            echo("即将创建 zhei 环境，请在下方输入 Python 版本号，默认为 3.9：", "yellow")
            python_version = input()
            echo("即将创建 zhei 环境，请在下方输入环境名称，将会自动安装 zhei 包，默认名称为 zhei：", "yellow")
            env_name = input()
            zhei = [
                "conda create -n {} python={}".format(env_name, python_version),
                "conda activate {}".format(env_name),
                "pip install zhei --upgrade",
            ]
            run_cmd(zhei)

            echo("即将安装 Pytorch，请选择 CUDA 版本号，默认为 11.8：", "yellow")
            cuda_version = input()
            torch_url, torchvision_url, torchaudio_url = show_available_version(cuda_version, python_version)

            echo("即将从以下链接安装 torch：\n {}".format(torch_url))
            torch_install = [
                "pip install {}".format(torch_url),
            ]   
            run_cmd(torch_install)

            echo("即将从以下链接安装 torchvision：\n {}".format(torchvision_url))  
            torchvision_install = [
                "pip install {}".format(torchvision_url),
            ]
            run_cmd(torchvision_install)

            echo("即将从以下链接安装 torchaudio：\n {}".format(torchaudio_url))
            torchaudio_install = [
                "pip install {}".format(torchaudio_url),
            ]   
            run_cmd(torchaudio_install)

        elif step == "5":
            run_cmd([
                "cd ~/",
                "wget https://download.redis.io/redis-stable.tar.gz",
                "tar -xzvf redis-stable.tar.gz",
                "cd redis-stable",
                "make",
                "cd src",
                "make install PREFIX=~/redis",
                "cp {}/redis.conf ~/redis/bin/".format(pkg_current_path),
                "~/redis/bin/redis-server ~/redis/bin/redis.conf",
            ])
            
        elif step == "6":
            run_cmd([
                "cd ~/.ssh",
                "ssh-keygen -t rsa",
                "cat id_rsa.pub"
            ])
            
        elif step == "7":
            target_path = input("请输入项目路径（包含新的项目文件夹名），默认为 ~/code/zhei_project：")
            run_cmd([
                "cp -r {}/ProjectTemplate {}".format(pkg_current_path, target_path),        
            ])
            
        elif step == "0":
            exit()

        else:
            continue
        
        



from bs4 import BeautifulSoup
import requests as rq
import pkg_resources
import os
from rich.console import Console

console = Console()

def package_installed(package_name):
    """
    检查指定包是否已安装
    """
    try:
        pkg_resources.get_distribution(package_name)
    except pkg_resources.DistributionNotFound:
        return False
    else:
        return True

def echo(msg, color="green"):
    console.print(msg, style=color)

def run_cmd(cmd_list):
    if not isinstance(cmd_list, list):
        cmd_list = [cmd_list]
    outputs = []
    for cmd in cmd_list:
        while True:
            exitcode, output = subprocess.getstatusoutput(cmd)
            if exitcode != 0:
                echo("执行 {} 失败！".format(cmd), "#FF6AB3")
                echo("可通过在下方修改命令继续执行，或者直接按下回车键忽略该错误：")
                cmd = input()
                if cmd == "":
                    outputs.append(output)
                    break
            else:
                outputs.append(output)
                break

    return outputs
        


def show_available_version(cuda_version="11.8", python_version="3.9"):
    url = "https://download.pytorch.org/whl/torch_stable.html"
    html = rq.get(url).text
    soup = BeautifulSoup(html, "html.parser")
    available_items = []
    backup_items = []
    for i, item in enumerate(soup.find_all("a")):
        if "-linux_x86_64.whl" in item.attrs["href"] and "cpu" not in item.attrs["href"]:
            if "/torch-" in item.attrs["href"] or "/torchvision-" in item.attrs["href"] or "/torchaudio-" in item.attrs["href"]:
                if item.attrs["href"].startswith("cu"):
                    backup_items.append(item)
                    if "-cp{}-".format(python_version.replace(".", "")) in item.attrs["href"]:
                        if "cu{}".format(cuda_version.replace(".", "")) in item.attrs["href"] and "-cp{}-".format(python_version.replace(".", "")) in item.attrs["href"]:
                            available_items.append(item)
        
    if len(available_items) == 0:
        print("未找到可用的版本！")
        print("可选的 CUDA 版本号：\n", set([item.attrs["href"].split("/")[0].replace("cu", "") for item in backup_items]))
        print("可选的 Python 版本号：\n", set([item.attrs["href"].split("-")[2].replace("cp", "") for item in backup_items]))
        cuda_version = input("请重新输入兼容的 CUDA 版本号：")
        python_version = input("请重新输入兼容的 Python 版本号：")
        return show_available_version(cuda_version, python_version)
    
    torch_version = []
    torchvision_version = []
    torchaudio_version = []
    for item in available_items:
        if "torch-" in item.attrs["href"]:
            torch_version.append(item.attrs["href"].split("-")[1].split("%")[0])
        if "torchvision-" in item.attrs["href"]:
            torchvision_version.append(item.attrs["href"].split("-")[1].split("%")[0])
        if "torchaudio-" in item.attrs["href"]:
            torchaudio_version.append(item.attrs["href"].split("-")[1].split("%")[0])
    
    echo("可选的 torch 版本号：")
    for i, item in enumerate(set(torch_version)):
        print("{}、 {}".format(i + 1, item))
    selected_torch_version = input("请选择 torch 版本号，输入序号即可：")
    selected_torch_version = list(set(torch_version))[int(selected_torch_version) - 1]
    print("可选的 torchvision 版本号：")
    for i, item in enumerate(set(torchvision_version)):
        print("{}、 {}".format(i + 1, item))
    selected_torchvision_version = input("请选择 torchvision 版本号，输入序号即可：")
    selected_torchvision_version = list(set(torchvision_version))[int(selected_torchvision_version) - 1]
    print("可选的 torchaudio 版本号：")
    for i, item in enumerate(set(torchaudio_version)):
        print("{}、 {}".format(i + 1, item))
    selected_torchaudio_version = input("请选择 torchaudio 版本号，输入序号即可：")
    selected_torchaudio_version = list(set(torchaudio_version))[int(selected_torchaudio_version) - 1]
    
    torch_url = ""
    torchvision_url = ""
    torchaudio_url = ""
    for i, item in enumerate(available_items):
        if "/torch-{}".format(selected_torch_version) in item.attrs["href"]:
            torch_url = "https://download.pytorch.org/whl/" + item.attrs["href"]
        if "/torchvision-{}".format(selected_torchvision_version) in item.attrs["href"]:
            torchvision_url = "https://download.pytorch.org/whl/" + item.attrs["href"]
        if "/torchaudio-{}".format(selected_torchaudio_version) in item.attrs["href"]:
            torchaudio_url = "https://download.pytorch.org/whl/" + item.attrs["href"]
    
    return torch_url, torchvision_url, torchaudio_url



import os
from zheinit.utils import run_cmd, echo
import yaml
import datetime
import time


def read_yaml(path):
    with open(path, 'r') as f:
        data = yaml.load(f, Loader=yaml.SafeLoader)
    return data


def run():
    
    # project_path = "/Users/dengyifan/Desktop/Github仓库/zheinit/zheinit/ProjectTemplate"
    project_path = os.getcwd()
    echo("")
    echo("")
    echo("")
    echo("      _____                          _      ____                  _")
    echo("     |  __ \\                        | |    |  _ \\                | |")
    echo("     | |__) |____      _____ _ __ __| |    | |_) |_   _          | |")
    echo("     |  ___/ _ \\ \\ /\\ / / _ \\ '__/ _` |    |  _ <| | | |     _   | |")
    echo("     | |  | (_) \\ V  V /  __/ | | (_| |    | |_) | |_| |    | |__| |")
    echo("     |_|   \\___/ \\_/\\_/ \\___|_|  \\__,_|    |____/ \\__, |     \\____/")
    echo("                                                   __/ |")
    echo("                                                  |___/")
    echo("")
    echo("")
    echo("")
    echo("                      欢迎使用 zhei 项目启动器！")
    echo("")
    echo("当前工作目录为：{}".format(project_path))
    
    # 获取可选的项目名
    projects = os.listdir(os.path.join(project_path, "configs/exps"))
    echo("\n可选的项目名：")
    for i, project in enumerate(projects):
        if "." in project:
            continue
        echo("{}、 {}".format(i + 1, project), "#FF6AB3")
    
    echo("\n请在下方输入您的项目名或直接选择序号：")
    project_name = input()
    if project_name.isdigit():
        project_name = projects[int(project_name) - 1]
    
    echo("\n选择启动 {} 项目计划的模式，目前支持 “nohup”、“tmux”、“python”，默认为 “nohup”：".format(project_name))
    start_mode = input()
    conda_env = ""
    if start_mode not in ["nohup", "tmux", "python"] or start_mode == "":
        start_mode = "nohup"
    if start_mode == "tmux":
        echo("\n请在下方输入您的 conda 环境名，默认 zhei：")
        conda_env = input("")
        if conda_env == "":
            conda_env = "zhei"
    
    # ---------------------------------------------------------------------------- #
    #                         获取实验计划                                     
    # ---------------------------------------------------------------------------- #
    exp_plan_path = os.path.join(project_path, "configs/exps/{}/exp_plan.yaml".format(project_name))
    exp_plan = read_yaml(exp_plan_path)
    
    # ---------------------------------------------------------------------------- #
    #                         获取默认配置                                     
    # ---------------------------------------------------------------------------- #
    defalut_exp_cfg = read_yaml(os.path.join(project_path, "configs/default_config.yaml"))
    defalut_exp_cfg_keys = list(defalut_exp_cfg.keys())
    
    
    exp_num = len(exp_plan['experiments'])
    echo("\n本次计划运行 {} 个实验".format(exp_num), "blue")
    
    # ---------------------------------------------------------------------------- #
    #                         逐个启动实验                                     
    # ---------------------------------------------------------------------------- #
    for i, exp_name in enumerate(exp_plan['experiments'].keys()):
        echo("\n正在启动第 {} 个实验：{}".format(i+1, exp_name), "yellow")
        # ---------------------------------------------------------------------------- #
        #                         获取实验配置                                     
        # ---------------------------------------------------------------------------- #
        exp_cfg_path = exp_plan['experiments'][exp_name]['config_path']
        hyper_params = exp_plan['experiments'][exp_name]['hyper_params']
        exp_cfg = read_yaml(os.path.join(project_path, "configs/exps/{}.yaml".format(exp_cfg_path)))
        exp_cfg_keys = list(exp_cfg.keys())
        all_existed_keys = list(set(defalut_exp_cfg_keys + exp_cfg_keys))
        
        # ---------------------------------------------------------------------------- #
        #                         组装命令行参数                                     
        # ---------------------------------------------------------------------------- #
        hyper_params_str = ""
        for hyper_param in hyper_params:
            hyper_params_value = "\"{}\"".format(hyper_params[hyper_param]) if isinstance(hyper_params[hyper_param], str) else hyper_params[hyper_param]
            
            if hyper_param not in all_existed_keys:
                hyper_params_str += " +{}={}".format(hyper_param, hyper_params_value)
            else:
                hyper_params_str += " {}={}".format(hyper_param, hyper_params_value)
        
    
        if start_mode == "nohup":
            log_path = os.path.join(project_path, "logs/{}/{}.log".format(project_name, "{}_{}".format(exp_name, datetime.datetime.now().strftime('%Y%m%d%H%M%S'))))
            cmd = "nohup python run.py"
            cmd += hyper_params_str
            cmd += " +experiments={}".format(exp_cfg_path)
            cmd += " > {} 2>&1 &".format(log_path)
            echo(cmd, "yellow")
            echo("查看日志：tail -f {}".format(log_path))
            run_cmd(cmd)
            
        elif start_mode == "python":
            cmd = "python run.py"
            cmd += hyper_params_str
            cmd += " +experiments={}".format(exp_cfg_path)
            echo(cmd, "yellow")
            run_cmd(cmd)
            
        elif start_mode == "tmux":
            tmux_session = "{}@{}".format(exp_name, datetime.datetime.now().strftime('%Y%m%d%H%M%S'))
            
            cmd = "python run.py"
            cmd += hyper_params_str
            cmd += " +tmux_session=\"{}\"".format(tmux_session)
            cmd += " +experiments={}".format(exp_cfg_path)
            
            run_cmd("tmux new-session -d -s {}".format(tmux_session))
            run_cmd("tmux send-keys -t {} {}".format(tmux_session, "cd {}".format(project_path)))
            run_cmd("tmux send-keys -t {} {}".format(tmux_session, "C-m"))
            run_cmd("tmux send-keys -t {} {}".format(tmux_session, "conda activate {}".format(conda_env)))
            run_cmd("tmux send-keys -t {} {}".format(tmux_session, "C-m"))
            run_cmd("tmux send-keys -t {} {}".format(tmux_session, cmd))
            run_cmd("tmux send-keys -t {} {}".format(tmux_session, "C-m"))
        
            echo(cmd, "yellow")
            echo("查看日志：tmux attach -t {}".format(tmux_session))
            echo("关闭日志：tmux kill-session -t {}".format(tmux_session))
            run_cmd(cmd)
            
        echo("实验 {} 已启动！".format(exp_name), "#F48671")
        time.sleep(5)

    
    echo("\n所有实验均已启动！祝好运！", "green")
    
# run()
